@@ -3,7 +3,7 @@
 # This file defines the default bindings for Tk scale widgets and provides
 # procedures that help in implementing the bindings.
 #
-# RCS: @(#) $Id: scale.tcl,v 1.3 1998/09/14 18:23:24 stanton Exp $
+# RCS: @(#) $Id: scale.tcl,v 1.6 2000/01/06 02:22:24 hobbs Exp $
 #
 # Copyright (c) 1994 The Regents of the University of California.
 # Copyright (c) 1994-1995 Sun Microsystems, Inc.
@@ -32,7 +32,7 @@ bind Scale <Leave> {
     if {$tk_strictMotif} {
 	%W config -activebackground $tkPriv(activeBg)
     }
-    if {[%W cget -state] == "active"} {
+    if {[string equal [%W cget -state] "active"]} {
 	%W configure -state normal
     }
 }
@@ -107,10 +107,10 @@ bind Scale <End> {
 
 proc tkScaleActivate {w x y} {
     global tkPriv
-    if {[$w cget -state] == "disabled"} {
-	return;
+    if {[string equal [$w cget -state] "disabled"]} {
+	return
     }
-    if {[$w identify $x $y] == "slider"} {
+    if {[string equal [$w identify $x $y] "slider"]} {
 	$w configure -state active
     } else {
 	$w configure -state normal
@@ -129,11 +129,11 @@ proc tkScaleButtonDown {w x y} {
     global tkPriv
     set tkPriv(dragging) 0
     set el [$w identify $x $y]
-    if {$el == "trough1"} {
+    if {[string equal $el "trough1"]} {
 	tkScaleIncrement $w up little initial
-    } elseif {$el == "trough2"} {
+    } elseif {[string equal $el "trough2"]} {
 	tkScaleIncrement $w down little initial
-    } elseif {$el == "slider"} {
+    } elseif {[string equal $el "slider"]} {
 	set tkPriv(dragging) 1
 	set tkPriv(initValue) [$w get]
 	set coords [$w coords]
@@ -158,8 +158,7 @@ proc tkScaleDrag {w x y} {
     if {!$tkPriv(dragging)} {
 	return
     }
-    $w set [$w get [expr {$x - $tkPriv(deltaX)}] \
-	    [expr {$y - $tkPriv(deltaY)}]]
+    $w set [$w get [expr {$x-$tkPriv(deltaX)}] [expr {$y-$tkPriv(deltaY)}]]
 }
 
 # tkScaleEndDrag --
@@ -194,7 +193,7 @@ proc tkScaleEndDrag {w} {
 proc tkScaleIncrement {w dir big repeat} {
     global tkPriv
     if {![winfo exists $w]} return
-    if {$big == "big"} {
+    if {[string equal $big "big"]} {
 	set inc [$w cget -bigincrement]
 	if {$inc == 0} {
 	    set inc [expr {abs([$w cget -to] - [$w cget -from])/10.0}]
@@ -205,19 +204,19 @@ proc tkScaleIncrement {w dir big repeat} {
     } else {
 	set inc [$w cget -resolution]
     }
-    if {([$w cget -from] > [$w cget -to]) ^ ($dir == "up")} {
+    if {([$w cget -from] > [$w cget -to]) ^ [string equal $dir "up"]} {
 	set inc [expr {-$inc}]
     }
     $w set [expr {[$w get] + $inc}]
 
-    if {$repeat == "again"} {
+    if {[string equal $repeat "again"]} {
 	set tkPriv(afterId) [after [$w cget -repeatinterval] \
-		tkScaleIncrement $w $dir $big again]
-    } elseif {$repeat == "initial"} {
+		[list tkScaleIncrement $w $dir $big again]]
+    } elseif {[string equal $repeat "initial"]} {
 	set delay [$w cget -repeatdelay]
 	if {$delay > 0} {
 	    set tkPriv(afterId) [after $delay \
-		    tkScaleIncrement $w $dir $big again]
+		    [list tkScaleIncrement $w $dir $big again]]
 	}
     }
 }
@@ -233,9 +232,9 @@ proc tkScaleIncrement {w dir big repeat} {
 
 proc tkScaleControlPress {w x y} {
     set el [$w identify $x $y]
-    if {$el == "trough1"} {
+    if {[string equal $el "trough1"]} {
 	$w set [$w cget -from]
-    } elseif {$el == "trough2"} {
+    } elseif {[string equal $el "trough2"]} {
 	$w set [$w cget -to]
     }
 }
@@ -252,8 +251,8 @@ proc tkScaleControlPress {w x y} {
 proc tkScaleButton2Down {w x y} {
     global tkPriv
 
-    if {[$w cget -state] == "disabled"} {
-	return;
+    if {[string equal [$w cget -state] "disabled"]} {
+      return
     }
     $w configure -state active
     $w set [$w get $x $y]
